﻿using UnityEngine;
using System.Collections;

// 雑魚敵処理
public class Enemy : EnemyCommon {

    public Transform groundCheck;

    public int count = 0;
    private float timer = 0f;
    public float MoveTime = 3f;
    public float walkForce = 50f;
    public float maxWalkSpeed = 3f;
    public float JumpForce = 800f;
    public float TimeSpeed = 1.0f;
    private int kind = 0;
    public bool Grounded = false; // 接地してるか否か
    private bool jump = false;
    private float jumpDir = 1;
    private bool breaking = false;

    void Start() {
        groundCheck = transform.FindChild("EnemyGroundCheck");
        count = 0;
        kind = 0;
        timer = MoveTime * TimeSpeed;
        jump = false;
        jumpDir = 1;
        breaking = false;
    }

    // Update is called once per frame
    void Update() {
        // 接地判定。「Ground」レイヤーとのみアタリ判定
        Vector3 pos0 = new Vector3(groundCheck.position.x, transform.position.y, transform.position.z);
        Grounded = Physics2D.Linecast(pos0, groundCheck.position, 1 << LayerMask.NameToLayer("Ground"));

        timer -= Time.deltaTime;
        switch (kind) {
            case 0:
                // 移動中
                if (!Grounded || timer <= 0f) {
                    breaking = !Grounded;
                    timer = 1f * TimeSpeed;
                    kind++;
                }
                break;
            case 1:
                // 停止中
                if (timer <= 0f) {
                    timer = 0.25f * TimeSpeed;
                    Flip();
                    kind++;
                }
                break;
            case 2:
                // 反転中
                if (timer <= 0f) {
                    timer = MoveTime * TimeSpeed;
                    kind = 0;
                }
                break;
            case 3:
                // 爆発
                if (timer <= 0f) {
                    Instantiate(ParticleBorn, transform.position, Quaternion.identity);
                    Destroy(gameObject);
                }
                break;
            default:
                break;
        }

        // ダメージ表現
        if (OnDamage || kind == 3) DispDamage();

        count++;
    }

    void FixedUpdate() {
        if (breaking) {
            rigidbody2D.velocity = new Vector2(0f, rigidbody2D.velocity.y);
            breaking = false;
        }

        if (kind == 0) {
            float h = (isRight) ? 1f : -1f;

            if (h * rigidbody2D.velocity.x < maxWalkSpeed)
                rigidbody2D.AddForce(Vector2.right * h * walkForce);

            // x方向の速度が最大速度を超えないようにする
            if (Mathf.Abs(rigidbody2D.velocity.x) > maxWalkSpeed)
                rigidbody2D.velocity = new Vector2(Mathf.Sign(rigidbody2D.velocity.x) * maxWalkSpeed, rigidbody2D.velocity.y);
        }

        if (jump) {
            rigidbody2D.AddForce(new Vector2(jumpDir * JumpForce / 2, JumpForce));
            jump = false;
        }
    }

    public override void SetDead(float h) {
        jumpDir = h;
        jump = true;
        timer = 0.5f;
        kind = 3;
    }
}
