﻿using UnityEngine;
using System.Collections;

// 雑魚敵ボール処理
public class EnemyBall : EnemyCommon {

    public float dxDef = 3f;
    public float dyDef = 5f;
    public float ay = 8f;
    public float deadDx = 5f;
    public float deadDy = 16f;
    public float gravi = -48f;
    public float deadTime = 0.4f;
    public float xofs = 9.8f;

    public Vector3 basePos;
    public Vector3 bornPos;
    public Vector3 spd;
    public float xmax;

    private int deadKind = 0;
    private float timer = 0f;

    // 最初のフレームで呼ばれる処理
    void Start() {
        bornPos = transform.localPosition;
        InitPosition();
        deadKind = 0;
    }

    // 毎フレーム呼ばれる処理
    void Update() {
        switch (deadKind) {
            case 0:
                // 通常処理
                transform.Translate(spd * Time.deltaTime);
                spd.y += ((transform.localPosition.y < basePos.y) ? ay : -ay) * Time.deltaTime;

                float x = transform.localPosition.x;
                if ((isRight && x > xmax) || (!isRight && x < xmax)) {
                    // 画面外に出たので消去
                    Destroy(gameObject);
                    // InitPosition();
                }
                break;
            case 1:
                // 死亡処理
                transform.Translate(spd * Time.deltaTime);
                spd.y += gravi * Time.deltaTime;
                timer -= Time.deltaTime;
                if (timer <= 0) {
                    // 爆発、かつ、消滅
                    Instantiate(ParticleBorn, transform.position, Quaternion.identity);
                    Destroy(gameObject);
                }
                break;
        }

        if (OnDamage) DispDamage();
    }

    // 初期位置を設定
    void InitPosition() {
        basePos = bornPos;

        // 画面内のプレイヤー位置で初期位置を設定
        GameObject pobj = GameObject.Find("player");
        if (pobj == null) Debug.LogError("Player Not found");

        Vector3 ppos = pobj.transform.localPosition;
        Vector3 cpos = Camera.main.transform.localPosition;
        Vector3 scale = transform.localScale;
        isRight = (ppos.x < cpos.x) ? false : true;
        if (isRight) {
            xmax = cpos.x + xofs;
            basePos.x = cpos.x - xofs;
            scale.x = -1f;
            spd = new Vector3(dxDef, dyDef, 0);
        }
        else {
            xmax = cpos.x - xofs;
            basePos.x = cpos.x + xofs;
            scale.x = 1f;
            spd = new Vector3(dxDef * -1, dyDef, 0);
        }
        transform.localPosition = basePos;
        transform.localScale = scale;
    }

    // 死亡デモ開始時に呼ばれる処理
    public override void SetDead(float h) {
        deadKind = 1;
        timer = deadTime;
        spd.x = deadDx * h;
        spd.y = deadDy;
    }
}
