﻿using UnityEngine;
using System.Collections;

// シーンが変わる際にフェードアウト・フェードインを行うクラス
//
// naichilab - Android iOSアプリ開発メモ: 【Unity】シーン遷移時のフェードイン・フェードアウトを実装してみた
// http://naichilab.blogspot.jp/2013/12/unity.html
//
public class FadeManager : SingletonMonoBehaviour<FadeManager> {

    /// <summary>暗転用黒テクスチャ</summary>
    private Texture2D blackTexture = null;
    /// <summary>フェード中の透明度</summary>
    private float fadeAlpha = 0;
    /// <summary>フェード中かどうか</summary>
    private bool isFading = false;

    public void OnGUI() {
        if (!isFading)
            return;

        if (blackTexture == null) {
            // 黒テクスチャ作成
            blackTexture = new Texture2D(32, 32, TextureFormat.RGB24, false);
            blackTexture.ReadPixels(new Rect(0, 0, 32, 32), 0, 0);
            blackTexture.SetPixel(0, 0, Color.white);
            blackTexture.Apply();
        }

        //透明度を更新して黒テクスチャを描画
        GUI.color = new Color(0, 0, 0, fadeAlpha);
        GUI.DrawTexture(new Rect(0, 0, Screen.width, Screen.height), blackTexture);
    }

    /// <summary>
    /// 画面遷移
    /// </summary>
    /// <param name='scene'>シーン名</param>
    /// <param name='interval'>暗転にかかる時間(秒)</param>
    public void LoadLevel(string scene, float interval) {
        StartCoroutine(TransScene(scene, interval));
    }


    /// <summary>
    /// シーン遷移用コルーチン
    /// </summary>
    /// <param name='scene'>シーン名</param>
    /// <param name='interval'>暗転にかかる時間(秒)</param>
    private IEnumerator TransScene(string scene, float interval) {
        //だんだん暗く
        isFading = true;
        float time = 0;
        while (time <= interval) {
            fadeAlpha = Mathf.Lerp(0f, 1f, time / interval);
            time += Time.deltaTime;
            yield return 0;
        }

        //シーン切替
        Application.LoadLevel(scene);

        //だんだん明るく
        time = 0;
        while (time <= interval) {
            fadeAlpha = Mathf.Lerp(1f, 0f, time / interval);
            time += Time.deltaTime;
            yield return 0;
        }

        isFading = false;
    }

}
