﻿using UnityEngine;
using System.Collections;

// カメラ追従しながらテクスチャ(マテリアル)のオフセットを変更して多重スクロール・縦方向版
public class ParallaxVertical : MonoBehaviour {

    public float scroll_speed = 1.0f;
    public float div_v = 20.0f;
    public bool ScrollEnable = true;

    // 表示する範囲
    public float disp_y_min = 0f;
    public float disp_y_max = 60f;

    private Vector3 oldcampos;
    private Vector3 oldpos;
    private Vector2 diff_v;
    private Vector2 old_tex_uv;


    // Use this for initialization
    void Start() {
        // 初期位置を覚えておく
        oldpos = transform.localPosition;
        oldcampos = Camera.main.transform.position;
        diff_v = new Vector2(oldcampos.x - oldpos.x, oldcampos.y - oldpos.y);
        Vector2 t = renderer.material.mainTextureOffset;
        old_tex_uv = new Vector2(t.x, t.y);
    }

    // Update is called once per frame
    void Update() {
        // カメラ座標と同じにして追従させる
        Camera camera = Camera.main;
        Vector3 cpos = camera.transform.position;
        transform.localPosition = new Vector3(oldpos.x, cpos.y - diff_v.y, oldpos.z);

        if (ScrollEnable) {
            // テクスチャオフセットをずらす量を算出
            float v = (cpos.y - oldcampos.y) / div_v;
            renderer.material.mainTextureOffset = new Vector2(old_tex_uv.x, old_tex_uv.y + v * scroll_speed);
        }

        // カメラ位置によって表示非表示を切り替える
        renderer.enabled = (disp_y_min <= cpos.y && cpos.y <= disp_y_max) ? true : false;
    }
}
