﻿using UnityEngine;
using System.Collections;

// プレイヤー処理
public class Player : MonoBehaviour {

    public string NextSceneName = "";

    public Transform groundCheck;
    public Transform groundCheckL;
    public Transform groundCheckR;

    // ステージ毎のアイテム数。ステージ毎に設定が必要
    public int ItemMax = 3;
    public int NowItem = 0;

    // 体力
    public int Life = 100;
    public int LifeMax = 100;
    public int OneDamage = 20; // 敵に触れた時のダメージ量

    // 移動速度
    public float walkForce = 50f;
    public float maxWalkSpeed = 8f;

    // ジャンプ時のパラメータ
    public float JumpForce = 1100f;

    // ダメージを受けた際の小ジャンプパラメータ
    public float DamageForceX = 800f;
    public float DamageForceY = 800f;
    public float GroundDamageV = 2.0f;
    public float DeadForceX = 1200f;
    public float DeadForceY = 2200f;

    public bool Jump = false;
    private bool isRight = true; // 右向きか否か
    private bool isAttack = false; // 攻撃中か否か
    public bool IsDamage = false; // ダメージを受けているか否か

    // 接地してるか否か
    public bool Grounded = false;
    public bool GroundedL = false;
    public bool GroundedR = false;

    public float FireBornOffset = 0.1f; // 弾発射位置のオフセット
    public float AttackWait = 0.1f;

    private int count = 0;
    private float timer = 0f;

    public int DeadKind = 0;
    public float DamageJump = 0f;
    public bool StageClear = false;

    // 火の玉発生用
    public GameObject FirePrefab;

    // スプライトレンダラー
    private SpriteRenderer sr;
    private SpriteRenderer deadSr;

    // アイテム取得時のパーティクルエフェクト
    public GameObject ParticleBorn;

    // 死亡時のパーティクルエフェクト
    public GameObject DeadParticle;

    public GameObject gwk;
    public GameObject fadeManager;

    // オブジェクト発生時に呼ばれる処理
    void Awake() {
        // 足元をチェックするための座標保持objを取得
        groundCheck = transform.Find("groundCheck");
        groundCheckL = transform.Find("groundCheckL");
        groundCheckR = transform.Find("groundCheckR");

        // renderer取得
        sr = gameObject.GetComponent<SpriteRenderer>();

        GameObject deadAnime = transform.FindChild("PlayerDeadAnime").gameObject;
        deadSr = deadAnime.GetComponent<SpriteRenderer>();
        deadSr.enabled = false;
    }

    // 最初の1フレーム目で呼ばれる処理
    void Start() {
        Time.timeScale = 1f;
        DeadKind = 0;
        StageClear = false;
        timer = 0f;
        deadSr.enabled = false;
        LoadLifeValue();
    }

    // 毎フレーム呼ばれる処理
    void Update() {
        // 接地判定。「Ground」レイヤーとのみアタリ判定
        int mask = 1 << LayerMask.NameToLayer("Ground");
        Grounded = Physics2D.Linecast(transform.position, groundCheck.position, mask);

        Vector3 t;
        t = new Vector3(groundCheckL.position.x, transform.position.y, transform.position.z);
        GroundedL = Physics2D.Linecast(t, groundCheckL.position, mask);

        t = new Vector3(groundCheckR.position.x, transform.position.y, transform.position.z);
        GroundedR = Physics2D.Linecast(t, groundCheckR.position, mask);

        if (DeadKind == 0 && !StageClear) {
            // 生きてる、かつ、ステージクリアしていない

            if (Input.GetButtonDown("Jump") && (Grounded || GroundedL || GroundedR)) {
                // ジャンプボタンが押されたのでジャンプフラグを立てる
                // 速度設定は、FixedUpdate()内で行う
                Jump = true;
            }

            if (Input.GetButtonDown("Fire1") && !isAttack) {
                // 攻撃ボタンが押された
                isAttack = true;

                // 火の玉を発生
                Vector3 pos = new Vector3(transform.position.x, transform.position.y + FireBornOffset, 1);
                GameObject fire = Instantiate(FirePrefab, pos, Quaternion.identity) as GameObject;
                fire.gameObject.SendMessage("setDirection", isRight);

                StartCoroutine("WaitForAttack");
            }

            if (IsDamage) {
                // ダメージ時の点滅処理
                float l = ((count & 0x2) == 0) ? 1.0f : 0.0f;
                sr.color = new Color(1f, 1f, 1f, l);
            }
        }

        // Animatorコンポーネントの各プロパティに値を設定
        GetComponent<Animator>().SetBool("Grounded", Grounded);
        GetComponent<Animator>().SetBool("Atacked", isAttack);
        GetComponent<Animator>().SetBool("Upup", (GroundedL || GroundedR) ? true : false);

        // 死亡時関係処理
        switch (DeadKind) {
            case 2:
                // 死亡モーションをAnimatorに反映、してたけど
                // 見た目の色々不都合があったので本体は使わないことにした
                // GetComponent<Animator>().SetBool("Deaded", true);

                sr.enabled = false; // 本体非表示
                deadSr.enabled = true; // 死亡デモ用Spriteを表示

                Time.timeScale = 0.3f; // スローモーションを設定
                timer = 0.75f;
                DeadKind++;
                break;
            case 3:
                // transform.Rotate(new Vector3(0, 0, 120 * Time.deltaTime));
                timer -= Time.deltaTime;
                if (timer <= 0f || Grounded) {
                    deadSr.enabled = false;
                    Time.timeScale = 1f;
                    Instantiate(DeadParticle, transform.position, Quaternion.identity);
                    timer = 2f;
                    DeadKind++;
                }
                break;
            case 4:
                timer -= Time.deltaTime;
                if (timer <= 0f) {
                    // Destroy(gameObject);
                    GoToGameover();
                    DeadKind++;
                }
                break;
            default:
                break;
        }

        count++;
    }

    // 物理演算絡みの処理
    void FixedUpdate() {
        float h;
        if (DeadKind == 0 && !StageClear)
            h = Input.GetAxisRaw("Horizontal"); // 左右キーの入力
        else
            h = 0f;

        if ((h > 0 && !isRight) || (h < 0 && isRight)) Flip();

        // 左右キーが入力されていたら歩きアニメをさせる
        bool mv = (h != 0) ? true : false;
        GetComponent<Animator>().SetBool("Moved", mv);
        if (h * rigidbody2D.velocity.x < maxWalkSpeed)
            rigidbody2D.AddForce(Vector2.right * h * walkForce);

        // x方向の速度が最大速度を超えないようにする
        if (Mathf.Abs(rigidbody2D.velocity.x) > maxWalkSpeed)
            rigidbody2D.velocity = new Vector2(Mathf.Sign(rigidbody2D.velocity.x) * maxWalkSpeed, rigidbody2D.velocity.y);

        if (Jump) {
            // ジャンプアニメを設定
            // anim.SetTrigger("Jump");

            // 速度を設定
            rigidbody2D.AddForce(new Vector2(0f, JumpForce));
            Jump = false;
        }

        switch (DeadKind) {
            case 0:
                // 通常時
                if (DamageJump > 0f) {
                    // ダメージ時の処理
                    // プレイヤーを後ろに飛ばす
                    float x = (transform.localScale.x >= 0) ? -1.0f : 1.0f;
                    rigidbody2D.AddForce(new Vector2(DamageForceX * x * DamageJump, DamageForceY * DamageJump));
                    DamageJump = 0;
                }
                break;
            case 1:
                // 死亡時のジャンプ
                rigidbody2D.AddForce(new Vector2(DeadForceX * ((transform.localScale.x >= 0) ? -1f : 1f), DeadForceY));
                DeadKind = 2;
                break;
            default:
                break;
        }
    }

    // 左右反転させる
    void Flip() {
        isRight = !isRight;
        Vector3 theScale = transform.localScale;
        theScale.x *= -1;
        transform.localScale = theScale;
    }

    // 何かと当たった際の処理(物理的衝突有)
    void OnCollisionEnter2D(Collision2D col) {
        if (IsDamage) return;

        switch (col.gameObject.tag) {
            case "Enemy":
                // 敵に当たった場合
                if (!(col.gameObject.GetComponent<EnemyCommon>().Deaded)) {
                    // 敵が生きてる
                    OnDamageEffect(1.0f, OneDamage);
                }
                break;
            case "Water":
                // 地形ダメージアタリに当たった場合
                OnDamageEffect(GroundDamageV, OneDamage);
                break;
            default:
                break;
        }
    }

    // 何かと当たった際の処理(物理的衝突無)
    void OnTriggerEnter2D(Collider2D col) {
        if (DeadKind > 0) return;

        switch (col.gameObject.tag) {
            case "Gate":
                // ステージクリアゲートに当たった場合
                if (NowItem >= ItemMax) {
                    GoToNextScene();
                    StageClear = true;
                }
                break;
            case "Treasure":
                // 宝箱に当たった場合
                NowItem++;
                Instantiate(ParticleBorn, col.gameObject.transform.position, Quaternion.identity);
                Destroy(col.gameObject); // 宝箱を消去
                break;
            case "Enemy":
                // 敵に当たった場合
                if (!IsDamage) {
                    if (!(col.gameObject.GetComponent<EnemyCommon>().Deaded)) {
                        // 敵が生きてる
                        OnDamageEffect(1.0f, OneDamage);
                    }
                }
                break;
            default:
                break;
        }
    }

    //　ダメージを受けた際の処理
    void OnDamageEffect(float v, int damageValue) {
        if (IsDamage) return;
        IsDamage = true;
        DamageJump = v;
        Life -= damageValue; // HPを減らす
        if (Life < 0) Life = 0;
        if (DeadKind == 0 && Life <= 0) {
            // HPが無くなったのでゲームオーバー処理に移行
            DeadKind = 1;
            IsDamage = false;
        }
        else {
            // コルーチン開始
            StartCoroutine("WaitForIt");
        }
    }

    IEnumerator WaitForIt() {
        // n秒間、処理を止める
        yield return new WaitForSeconds(3);

        // n秒後、ダメージフラグをfalseにして点滅を戻す
        IsDamage = false;
        sr.color = new Color(1f, 1f, 1f, 1f);
    }

    IEnumerator WaitForAttack() {
        yield return new WaitForSeconds(0.15f);

        isAttack = false;
    }

    // HP値をロード
    void LoadLifeValue() {
        if (GWk.Instance == null)
            Instantiate(gwk);

        Life = GWk.PlayerLife;
        LifeMax = GWk.PlayerLifeMax;
    }

    // ゲームオーバー画面に移行
    void GoToGameover() {
        if (FadeManager.Instance == null)
            Instantiate(fadeManager);

        // Application.LoadLevel("gameover");
        FadeManager.Instance.LoadLevel("gameover", 0.5f);
    }

    // 次のステージに移行
    void GoToNextScene() {
        if (GWk.Instance == null)
            Instantiate(gwk);

        GWk.PlayerLife = Life; // HP値を次のステージに持ち越す

        if (FadeManager.Instance == null)
            Instantiate(fadeManager);

        // Application.LoadLevel(NextSceneName);
        FadeManager.Instance.LoadLevel(NextSceneName, 0.5f);
    }
}
